// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using System.Reflection;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Tools;
using NinjaTrader.NinjaScript.DrawingTools;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.AddOns;

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.Sim22
{
	/// <summary>
	/// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
	/// Sim22 May 2016 NT8b11.
	/// Update V1.1: Moved input code to State.DataLoaded due to exceptions when placed in the market analyzer
	/// </summary>
	public class Sim22_Stochastics_Colored_AnyInputGLV1 : Indicator
	{
		#region Variables
		
		private double			ob	= 0.0;
		private double			os	= 0.0;
		
		private Series<double>	DUpperBand;
		private Series<double>	DLowerBand;
		private Series<double>	KUpperBand;
		private Series<double>	KLowerBand;
		
		private Series<double>		den;
		private Series<double>		fastK;
		private MIN					min;
		private MAX					max;
		private Series<double>		nom;
		private SMA					smaFastK;
		private SMA					smaK;
		private int					barNumberFromSessOpen;
		private double				gap = 0.0;
		private bool isPriceSeries	= true;
		
		#endregion

        protected override void OnStateChange()
        {
			if (State == State.SetDefaults)
			{
				Name = "Sim22_Stochastics_Colored_AnyInputGLV1.1";
                Description = "Intraday Stochastics colored that ignores price gaps at the start of the intraday session. Use any input including indicators. Sim22 May 2016 NT8b10.";
				IsOverlay				= false;
				DrawOnPricePanel 		= false;
				Calculate				= Calculate.OnPriceChange;
				DrawDRegion				= true;
				DrawKRegion				= false;
				OpacityDRegion			= 30;
				OpacityKRegion			= 30;
				PeriodD					= 7;
				PeriodK					= 14;
			    Smooth					= 3;
				UseGapless				= true;
				PlotPopsPoops 			= true;
				PlotReversals			= false;
				
				AddPlot(new Stroke(Brushes.LimeGreen, DashStyleHelper.Solid, 2f), PlotStyle.Line, "DUpper");
				AddPlot(new Stroke(Brushes.Gray, DashStyleHelper.Solid, 1f), PlotStyle.Line, "DMiddle");
				AddPlot(new Stroke(Brushes.Red, DashStyleHelper.Solid, 2f), PlotStyle.Line, "DLower");
				
				AddPlot(new Stroke(Brushes.LimeGreen, DashStyleHelper.Solid, 1f), PlotStyle.Line, "KUpper");
				AddPlot(new Stroke(Brushes.Gray, DashStyleHelper.Solid, 1f), PlotStyle.Line, "KMiddle");
				AddPlot(new Stroke(Brushes.Red, DashStyleHelper.Solid, 1f), PlotStyle.Line, "KLower");
			
				AddPlot(new Stroke(Brushes.Cyan, 4), PlotStyle.TriangleUp, "DReversalUp");
				AddPlot(new Stroke(Brushes.Magenta, 4), PlotStyle.TriangleDown, "DReversalDown");
				
				AddPlot(new Stroke(Brushes.LimeGreen, 4), PlotStyle.TriangleUp, "DPopUp");
				AddPlot(new Stroke(Brushes.Red, 4), PlotStyle.TriangleDown, "DPoopDown");

				///'K' is used for price markers only
				AddPlot(new Stroke(Brushes.Gray, DashStyleHelper.Solid, 1f), PlotStyle.Line, "D-PriceMarker");
				AddPlot(new Stroke(Brushes.Gray, DashStyleHelper.Solid, 1f), PlotStyle.Line, "K-PriceMarker");
				
				AddLine(new Stroke(Brushes.Red, DashStyleHelper.Dash, 1f), 20, "Lower");
				AddLine(new Stroke(Brushes.LimeGreen, DashStyleHelper.Dash, 1f), 80, "Upper");
			}
			else if (State == State.Configure)
			{
				den			= new Series<double>(this);
				nom			= new Series<double>(this);
				fastK		= new Series<double>(this);

				DUpperBand 	= new Series<double>(this, MaximumBarsLookBack.Infinite);
				DLowerBand 	= new Series<double>(this, MaximumBarsLookBack.Infinite);
				KUpperBand 	= new Series<double>(this, MaximumBarsLookBack.Infinite);
				KLowerBand 	= new Series<double>(this, MaximumBarsLookBack.Infinite);
				
				///Set limitations for plots
				ob = 	Plots[3].Min = Plots[0].Min = Lines[1].Value;
						Plots[4].Max = Plots[1].Max = Lines[1].Value;
						Plots[4].Min = Plots[1].Min = Lines[0].Value;
				os = 	Plots[5].Max = Plots[2].Max = Lines[0].Value;
			}
			else if (State == State.DataLoaded)
			{
				////Update 1.1. Moved code here due to exceptions raised when placed in a market analyzer
				/// 
				/// Instrument price type
				if (Input.GetType() == typeof(NinjaTrader.NinjaScript.PriceSeries))
				{
					isPriceSeries 	= true;
					min				= MIN(Low, PeriodK);
					max				= MAX(High, PeriodK);
					smaFastK		= SMA(fastK, Smooth);
					smaK			= SMA(KMiddle, PeriodD);
				}
				else
				/// Indicator
				{
					isPriceSeries 	= false;
					UseGapless 		= false;
					min				= MIN(Input, PeriodK);
					max				= MAX(Input, PeriodK);
					smaFastK		= SMA(fastK, Smooth);
					smaK			= SMA(KMiddle, PeriodD);
				}
			}
        }
		
		public override void OnCalculateMinMax()
		{    
			/// Maintains a fixed scale to show full range of stochastic.
			MinValue = 0.0;
		   	MaxValue = 100.00;
			if (PlotReversals)
			{
				MinValue -= Plots[3].Width * 2;
				MaxValue += Plots[4].Width * 2;
			}
		} 
		
		public override string DisplayName
		{
			get { return String.Format("Stoch" + (UseGapless ? "'GL'" : "") + "({0}/{1}/{2})", PeriodK, PeriodD, Smooth); }
		}
		
		
		#region 	FormatNumberDecimalPlaces
		
		Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum	dpEnum		= Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum.None;
		
		public override string FormatPriceMarker(double value)
		{
			/* using static class Sim22_PriceFormatter in the Addons folder */
			if (dpEnum == Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum.SameAsInstrument)
				return Sim22_PriceFormatter.FormatSameAsInstrumentPrice(value, TickSize);
			else
				return value.ToString("N" + Sim22_PriceFormatter.FormatNumberDecimalPlaces(dpEnum).ToString());
		}
		
		#endregion
		
		
		protected override void OnBarUpdate()
		{
			try
			{
				double input = Input[0];
				
				double partialGap	= 0.0;
				double min0 = min[0];

				if (UseGapless && IsFirstTickOfBar)
				{
					if (Bars.IsFirstBarOfSession &&  CurrentBar > 0 )
					{
						barNumberFromSessOpen = CurrentBar;
						gap = Open[0] - Close[1];
					}
					
					if ((CurrentBar - barNumberFromSessOpen) < PeriodK)
					{
						partialGap = gap * (PeriodK - (CurrentBar - barNumberFromSessOpen + 1))/PeriodK;
					}
				}

				nom[0]		= input - min0  - 2 * partialGap;
				den[0]		= max[0] - min0 - partialGap;
			
				if (den[0].ApproxCompare(0) == 0)
					fastK[0] = CurrentBar == 0 ? 50 : fastK[1];
				else
					fastK[0] = Math.Min(100, Math.Max(0, 100 * nom[0] / den[0]));

				KLower[0] = KMiddle[0] = KUpper[0] = smaFastK[0];
				DLower[0] = DMiddle[0] = DUpper[0] = smaK[0];													
				
				if (PaintPriceMarkers && CurrentBar == (Bars.Count - 1))
				{
					D[0] = smaK[0];
					
					if (D[0] >= ob)
						PlotBrushes[10][0] = Plots[0].Brush;
					else if (D[0] <= os)
						PlotBrushes[10][0] = Plots[2].Brush;
					else
						PlotBrushes[10][0] = Plots[1].Brush;
					
					K[0] = smaFastK[0];	
					
					if (K[0] >= ob)
						PlotBrushes[11][0] = Plots[3].Brush;
					else if (K[0] <= os)
						PlotBrushes[11][0] = Plots[5].Brush;
					else
						PlotBrushes[11][0] = Plots[4].Brush;
				}
				
				if (DrawDRegion)
				{
					DUpperBand[0] = DUpper[0] >= ob ? DUpper[0] : ob;
					DLowerBand[0] = DLower[0] <= os ? DLower[0] : os;
					
					Draw.Region(this,"upperD", CurrentBar, 0, DUpperBand, ob, Plots[0].Brush, OpacityDRegion);
					Draw.Region(this,"lowerD", CurrentBar, 0, DLowerBand, os, Plots[2].Brush, OpacityDRegion);
				}
				
				if (DrawKRegion)
				{
					KUpperBand[0] = KUpper[0] >= ob ? KUpper[0] : ob;
					KLowerBand[0] = KLower[0] <= os ? KLower[0] : os;
					
					Draw.Region(this,"upperK", CurrentBar, 0, KUpperBand, ob, Plots[3].Brush, OpacityKRegion);
					Draw.Region(this,"lowerK", CurrentBar, 0, KLowerBand, os, Plots[5].Brush, OpacityKRegion);
				}
				
				if (base.IsFirstTickOfBar)
				{
					if (PlotReversals && CurrentBar > 3)
					{
						if ((IsRising(DLower)) && ((DLower[1]) < os) && (DLower[2]) >= (DLower[1]))
						{
							DReversalUp[1] = DUpper[1];
						}
						
						if ((IsFalling(DUpper)) && ((DUpper[1]) > ob) && (DUpper[2]) <= (DUpper[1]))
						{
							DReversalDown[1] = DLower[1];
						}
					}
				}
				
				if (PlotPopsPoops && CurrentBar > 3)
				{
					if (CrossAbove(DUpper, ob, 1))
					{
						DPopUp[0] = ob;
					}
					
					if (CrossBelow(DLower, os, 1))
					{
						DPoopDown[0] = os;
					}
				}
			}
			catch{}
		}

		#region Properties
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DUpper
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DMiddle
		{
			get { return Values[1]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DLower
		{
			get { return Values[2]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> KUpper
		{
			get { return Values[3]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> KMiddle
		{
			get { return Values[4]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> KLower
		{
			get { return Values[5]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DReversalUp
		{
			get { return Values[6]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DReversalDown
		{
			get { return Values[7]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DPopUp
		{
			get { return Values[8]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> DPoopDown
		{
			get { return Values[9]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> D
		{
			get { return Values[10]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> K
		{
			get { return Values[11]; }
		}
		
		[NinjaScriptProperty]
		[Display(Name="Decimal Places", Description="", Order=0, GroupName="Set up")]
		public  Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum DPEnum
		{
			get { return dpEnum; }
			set { dpEnum = value; }
		}
		
		[NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "(Intraday) Plot gapless?", GroupName = "NinjaScriptParameters", Order = 0)]
		public bool UseGapless
		{ get; set; }
		
		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "PeriodD", GroupName = "NinjaScriptParameters", Order = 1)]
		public int PeriodD
		{ get; set; }

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "PeriodK", GroupName = "NinjaScriptParameters", Order = 2)]
		public int PeriodK
		{ get; set; }

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Smooth", GroupName = "NinjaScriptParameters", Order = 3)]
		public int Smooth
		{ get; set; }
		
		[NinjaScriptProperty]		
		[Display(Name = "Plot D reversals?", Description = "Plot reversals?", GroupName = ".Plots", Order = 0)]		
		public bool PlotReversals
		{ get; set; }
		
		[NinjaScriptProperty]		
		[Display(Name = "Plot D pops & poops?", Description = "Plot pops & poops?", GroupName = ".Plots", Order = 1)]		
		public bool PlotPopsPoops
		{ get; set; }
		
		[NinjaScriptProperty]		
		[Display(Name = "Shade D region?", Description = "Shade D region?", GroupName = ".Plots", Order = 3)]	
		public bool DrawDRegion
		{ get; set; }
		
		[Range(0, 100), NinjaScriptProperty]		
		[Display(Name = "D region opacity (0-100)", Description = "D region opacity (0-100)", GroupName = ".Plots", Order = 4)]		
		public int OpacityDRegion
		{ get; set; }
		
		[NinjaScriptProperty]		
		[Display(Name = "Shade K region?", Description = "Shade K region?", GroupName = ".Plots", Order = 5)]		
		public bool DrawKRegion
		{ get; set; }
		
		[Range(0, 100), NinjaScriptProperty]		
		[Display(Name = "K region opacity (0-100)", Description = "K region opacity (0-100)", GroupName = ".Plots", Order = 6)]	
		public int OpacityKRegion
		{ get; set; }
		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Sim22.Sim22_Stochastics_Colored_AnyInputGLV1[] cacheSim22_Stochastics_Colored_AnyInputGLV1;
		public Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			return Sim22_Stochastics_Colored_AnyInputGLV1(Input, dPEnum, useGapless, periodD, periodK, smooth, plotReversals, plotPopsPoops, drawDRegion, opacityDRegion, drawKRegion, opacityKRegion);
		}

		public Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(ISeries<double> input, Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			if (cacheSim22_Stochastics_Colored_AnyInputGLV1 != null)
				for (int idx = 0; idx < cacheSim22_Stochastics_Colored_AnyInputGLV1.Length; idx++)
					if (cacheSim22_Stochastics_Colored_AnyInputGLV1[idx] != null && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].DPEnum == dPEnum && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].UseGapless == useGapless && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].PeriodD == periodD && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].PeriodK == periodK && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].Smooth == smooth && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].PlotReversals == plotReversals && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].PlotPopsPoops == plotPopsPoops && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].DrawDRegion == drawDRegion && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].OpacityDRegion == opacityDRegion && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].DrawKRegion == drawKRegion && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].OpacityKRegion == opacityKRegion && cacheSim22_Stochastics_Colored_AnyInputGLV1[idx].EqualsInput(input))
						return cacheSim22_Stochastics_Colored_AnyInputGLV1[idx];
			return CacheIndicator<Sim22.Sim22_Stochastics_Colored_AnyInputGLV1>(new Sim22.Sim22_Stochastics_Colored_AnyInputGLV1(){ DPEnum = dPEnum, UseGapless = useGapless, PeriodD = periodD, PeriodK = periodK, Smooth = smooth, PlotReversals = plotReversals, PlotPopsPoops = plotPopsPoops, DrawDRegion = drawDRegion, OpacityDRegion = opacityDRegion, DrawKRegion = drawKRegion, OpacityKRegion = opacityKRegion }, input, ref cacheSim22_Stochastics_Colored_AnyInputGLV1);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			return indicator.Sim22_Stochastics_Colored_AnyInputGLV1(Input, dPEnum, useGapless, periodD, periodK, smooth, plotReversals, plotPopsPoops, drawDRegion, opacityDRegion, drawKRegion, opacityKRegion);
		}

		public Indicators.Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(ISeries<double> input , Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			return indicator.Sim22_Stochastics_Colored_AnyInputGLV1(input, dPEnum, useGapless, periodD, periodK, smooth, plotReversals, plotPopsPoops, drawDRegion, opacityDRegion, drawKRegion, opacityKRegion);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			return indicator.Sim22_Stochastics_Colored_AnyInputGLV1(Input, dPEnum, useGapless, periodD, periodK, smooth, plotReversals, plotPopsPoops, drawDRegion, opacityDRegion, drawKRegion, opacityKRegion);
		}

		public Indicators.Sim22.Sim22_Stochastics_Colored_AnyInputGLV1 Sim22_Stochastics_Colored_AnyInputGLV1(ISeries<double> input , Sim22_PriceFormatter.FormatNumberDecimalPlacesEnum dPEnum, bool useGapless, int periodD, int periodK, int smooth, bool plotReversals, bool plotPopsPoops, bool drawDRegion, int opacityDRegion, bool drawKRegion, int opacityKRegion)
		{
			return indicator.Sim22_Stochastics_Colored_AnyInputGLV1(input, dPEnum, useGapless, periodD, periodK, smooth, plotReversals, plotPopsPoops, drawDRegion, opacityDRegion, drawKRegion, opacityKRegion);
		}
	}
}

#endregion
